@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2008-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: notify.c 11387 2010-11-06 14:37:34Z charles $
+ * $Id: notify.c 11709 2011-01-19 13:48:47Z jordan $
  */
 
 #include <string.h>
@@ -29,15 +29,15 @@
 
 #ifndef HAVE_LIBNOTIFY
 
-void tr_notify_init( void ) { }
-void tr_notify_send( TrTorrent * tor UNUSED ) { }
-void tr_notify_added( const char * name UNUSED ) { }
+void gtr_notify_init( void ) { }
+void gtr_notify_send( TrTorrent * tor UNUSED ) { }
+void gtr_notify_added( const char * name UNUSED ) { }
 
 #else
  #include <libnotify/notify.h>
 
 void
-tr_notify_init( void )
+gtr_notify_init( void )
 {
     notify_init( "Transmission" );
 }
@@ -109,10 +109,10 @@ addIcon( NotifyNotification * notify )
 }
 
 void
-tr_notify_send( TrTorrent *tor )
+gtr_notify_send( TrTorrent *tor )
 {
 #ifdef HAVE_LIBCANBERRA
-    if( pref_flag_get( PREF_KEY_PLAY_DOWNLOAD_COMPLETE_SOUND ) )
+    if( gtr_pref_flag_get( PREF_KEY_PLAY_DOWNLOAD_COMPLETE_SOUND ) )
     {
         /* play the sound, using sounds from the naming spec */
         ca_context_play( ca_gtk_context_get (), 0,
@@ -123,7 +123,7 @@ tr_notify_send( TrTorrent *tor )
     }
 #endif
 
-    if( pref_flag_get( PREF_KEY_SHOW_DESKTOP_NOTIFICATION ) )
+    if( gtr_pref_flag_get( PREF_KEY_SHOW_DESKTOP_NOTIFICATION ) )
     {
         const tr_info * info = tr_torrent_info( tor );
         NotifyNotification * n;
@@ -155,12 +155,17 @@ tr_notify_send( TrTorrent *tor )
 }
 
 void
-tr_notify_added( const char * name )
+gtr_notify_added( const char * name )
 {
-    if( pref_flag_get( PREF_KEY_SHOW_DESKTOP_NOTIFICATION ) )
+    if( gtr_pref_flag_get( PREF_KEY_SHOW_DESKTOP_NOTIFICATION ) )
     {
         NotifyNotification * n = notify_notification_new(
-            _( "Torrent Added" ), name, NULL, NULL );
+            _( "Torrent Added" ), name, NULL
+/* the fourth argument was removed in libnotify 0.7.0 */
+#if !defined(NOTIFY_VERSION_MINOR) || (NOTIFY_VERSION_MAJOR == 0 && NOTIFY_VERSION_MINOR < 7)
+	    , NULL
+#endif
+	    );
         addIcon( n );
         notify_notification_set_timeout( n, NOTIFY_EXPIRES_DEFAULT );
         notify_notification_show( n, NULL );
