@@ -1,22 +1,21 @@
 /*
- * This file Copyright (C) 2007-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: tr-prefs.h 11128 2010-08-06 06:22:05Z charles $
+ * $Id: tr-prefs.h 11709 2011-01-19 13:48:47Z jordan $
  */
 
-#ifndef TR_PREFS_H
-#define TR_PREFS_H
+#ifndef GTR_PREFS_H
+#define GTR_PREFS_H
 
 #include <gtk/gtk.h>
 
-GtkWidget * tr_prefs_dialog_new( GObject *   core,
-                                 GtkWindow * parent );
+GtkWidget * gtr_prefs_dialog_new( GtkWindow * parent, GObject * core );
 
 /* if you add a key here,  you /must/ add its
  * default in tr_prefs_init_defaults( void ) */
@@ -55,4 +54,4 @@ enum
     SECONDARY_WINDOW_REFRESH_INTERVAL_SECONDS = 2
 };
 
-#endif
+#endif /* GTR_PREFS_H */
