@@ -69,7 +69,7 @@ SOFTWARE.
 #include <locale.h>
 
 #include <stdarg.h> /* some 1.4.x versions of evutil.h need this */
-#include <evutil.h> /* evutil_strtoll() */
+#include <event2/util.h> /* evutil_strtoll() */
 
 #include "JSON_parser.h"
 
