@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: platform.h 11126 2010-08-05 19:54:44Z charles $
+ * $Id: platform.h 11709 2011-01-19 13:48:47Z jordan $
  */
 
 #ifndef __TRANSMISSION__
@@ -54,6 +54,10 @@ const char * tr_getTorrentDir( const tr_session * );
 /** @brief return the directory where the Web Client's web ui files are kept */
 const char * tr_getWebClientDir( const tr_session * );
 
+/** @brief return the number of bytes available for use in the specified path, or -1 on error */
+int64_t tr_getFreeSpace( const char * path );
+
+
 /** @} */
 
 
