@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: StatusBarView.m 11489 2010-12-05 19:20:07Z livings124 $
+ * $Id: StatusBarView.m 11625 2011-01-03 03:41:42Z livings124 $
  * 
- * Copyright (c) 2006-2010 Transmission authors and contributors
+ * Copyright (c) 2006-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,55 +24,88 @@
 
 #import "StatusBarView.h"
 
+@interface StatusBarView (Private)
+
+- (void) reload;
+
+@end
+
 @implementation StatusBarView
 
 - (id) initWithFrame: (NSRect) rect
 {
     if ((self = [super initWithFrame: rect]))
     {
-        fGrayBorderColor = [[NSColor colorWithCalibratedRed: 171.0/255.0 green: 171.0/255.0 blue: 171.0/255.0 alpha: 1.0] retain];
+        NSColor * lightColor = [NSColor colorWithCalibratedRed: 160.0/255.0 green: 160.0/255.0 blue: 160.0/255.0 alpha: 1.0];
+        NSColor * darkColor = [NSColor colorWithCalibratedRed: 155.0/255.0 green: 155.0/255.0 blue: 155.0/255.0 alpha: 1.0];
+        fGradient = [[NSGradient alloc] initWithStartingColor: lightColor endingColor: darkColor];
+        
+        [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(reload)
+            name: NSWindowDidBecomeMainNotification object: [self window]];
+        [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(reload)
+            name: NSWindowDidResignMainNotification object: [self window]];
     }
     return self;
 }
 
 - (void) dealloc
 {
-    [fGrayBorderColor release];
+    [fGradient release];
     [super dealloc];
 }
 
 - (void) drawRect: (NSRect) rect
 {
+    const BOOL active = [[self window] isMainWindow];
+    
     NSInteger count = 0;
-    NSRect gridRects[3];
-    NSColor * colorRects[3];
+    NSRect gridRects[active ? 2 : 3];
+    NSColor * colorRects[active ? 2 : 3];
     
     NSRect lineBorderRect = NSMakeRect(NSMinX(rect), NSHeight([self bounds]) - 1.0, NSWidth(rect), 1.0);
-    if (NSIntersectsRect(lineBorderRect, rect))
+    if (active)
     {
-        gridRects[count] = lineBorderRect;
-        colorRects[count] = [NSColor whiteColor];
-        ++count;
-        
-        rect.size.height -= 1.0;
+        if (NSIntersectsRect(lineBorderRect, rect))
+        {
+            gridRects[count] = lineBorderRect;
+            colorRects[count] = [NSColor colorWithCalibratedWhite: 0.75 alpha: 1.0];
+            ++count;
+            
+            rect.size.height -= 1.0;
+        }
     }
     
     lineBorderRect.origin.y = 0.0;
     if (NSIntersectsRect(lineBorderRect, rect))
     {
         gridRects[count] = lineBorderRect;
-        colorRects[count] = fGrayBorderColor;
+        colorRects[count] = active ? [NSColor colorWithCalibratedWhite: 0.25 alpha: 1.0]
+                                    : [NSColor colorWithCalibratedWhite: 0.5 alpha: 1.0];
         ++count;
         
         rect.origin.y += 1.0;
         rect.size.height -= 1.0;
     }
     
-    gridRects[count] = rect;
-    colorRects[count] = [NSColor controlColor];
-    ++count;
+    if (active)
+        [fGradient drawInRect: rect angle: 270.0];
+    else
+    {
+        gridRects[count] = rect;
+        colorRects[count] = [NSColor colorWithCalibratedWhite: 0.85 alpha: 1.0];
+        ++count;
+    }
     
     NSRectFillListWithColors(gridRects, colorRects, count);
 }
 
 @end
+
+@implementation StatusBarView (Private)
+
+- (void) reload
+{
+    [self setNeedsDisplay: YES];
+}
+
+@end
