@@ -37,7 +37,7 @@
 <context>
     <name>Details</name>
     <message>
-        <location filename="../details.cc" line="+151"/>
+        <location filename="../details.cc" line="+153"/>
         <source>Torrent Properties</source>
         <translation>Propriedades do Torrent</translation>
     </message>
@@ -67,7 +67,7 @@
         <translation>Opções</translation>
     </message>
     <message>
-        <location line="+144"/>
+        <location line="+176"/>
         <source>None</source>
         <translation>Nenhum</translation>
     </message>
@@ -78,12 +78,12 @@
     </message>
     <message>
         <location line="+1"/>
-        <location line="+181"/>
+        <location line="+175"/>
         <source>Unknown</source>
         <translation>Desconhecido</translation>
     </message>
     <message>
-        <location line="-149"/>
+        <location line="-143"/>
         <source>Finished</source>
         <translation>Concluído</translation>
     </message>
@@ -93,17 +93,15 @@
         <translation>Pausado</translation>
     </message>
     <message>
-        <location line="+42"/>
         <source>%1 (%2%)</source>
-        <translation>%1 (%2%)</translation>
+        <translation type="obsolete">%1 (%2%)</translation>
     </message>
     <message>
-        <location line="+46"/>
         <source>%1 (+%2 corrupt)</source>
-        <translation>%1 (+%2 corrompidos)</translation>
+        <translation type="obsolete">%1 (+%2 corrompidos)</translation>
     </message>
     <message>
-        <location line="+81"/>
+        <location line="+163"/>
         <source>Active now</source>
         <translation>Ativo</translation>
     </message>
@@ -154,7 +152,7 @@
         <translation>Criado pelo %1 em %2</translation>
     </message>
     <message>
-        <location line="+139"/>
+        <location line="+121"/>
         <location line="+21"/>
         <source>Encrypted connection</source>
         <translation>Conexão criptografada</translation>
@@ -215,9 +213,8 @@
         <translation>Atividade</translation>
     </message>
     <message>
-        <location line="+1"/>
         <source>Torrent size:</source>
-        <translation>Tamanho:</translation>
+        <translation type="obsolete">Tamanho:</translation>
     </message>
     <message>
         <location line="+1"/>
@@ -270,7 +267,7 @@
         <translation>Detalhes</translation>
     </message>
     <message>
-        <location line="+1"/>
+        <location line="+2"/>
         <source>Location:</source>
         <translation>Local:</translation>
     </message>
@@ -295,7 +292,7 @@
         <translation>Comentário:</translation>
     </message>
     <message>
-        <location line="+125"/>
+        <location line="+105"/>
         <location line="+12"/>
         <location line="+34"/>
         <source>Error</source>
@@ -307,17 +304,27 @@
         <translation>O rastreador já existe</translation>
     </message>
     <message>
-        <location line="-614"/>
-        <source>%1 (%2% of %3% Available)</source>
+        <location line="-581"/>
+        <source>%1 (100%)</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <location line="+5"/>
+        <source>%1 of %2 (%3%)</source>
+        <translation type="unfinished">%1 de %2 (%3%)</translation>
+    </message>
     <message>
         <location line="+7"/>
-        <source>%1 (%2% of %3% Available) + %4 Unverified</source>
+        <source>%1 of %2 (%3%), %4 Unverified</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location line="+48"/>
+        <location line="+33"/>
+        <source>%1 (%2 corrupt)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+15"/>
         <source>%1 (Ratio: %2)</source>
         <translation type="unfinished"></translation>
     </message>
@@ -327,7 +334,12 @@
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location line="+357"/>
+        <location line="+217"/>
+        <source>Size:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+102"/>
         <source>Add URL </source>
         <translation type="unfinished"></translation>
     </message>
@@ -368,12 +380,12 @@
         <translation>Limitar velocidade de download (%1):</translation>
     </message>
     <message>
-        <location line="+11"/>
+        <location line="+12"/>
         <source>Limit &amp;upload speed (%1):</source>
         <translation>Limitar velocidade de upload (%1):</translation>
     </message>
     <message>
-        <location line="+12"/>
+        <location line="+13"/>
         <source>High</source>
         <translation>Alta</translation>
     </message>
@@ -399,12 +411,12 @@
     </message>
     <message>
         <location line="+5"/>
-        <location line="+14"/>
+        <location line="+15"/>
         <source>Use Global Settings</source>
         <translation>Usar Configurações Globais</translation>
     </message>
     <message>
-        <location line="-13"/>
+        <location line="-14"/>
         <source>Seed regardless of ratio</source>
         <translation>Semear independente da proporção</translation>
     </message>
@@ -414,7 +426,7 @@
         <translation>Parar de semear na proporção:</translation>
     </message>
     <message>
-        <location line="+7"/>
+        <location line="+8"/>
         <source>&amp;Ratio:</source>
         <translation>Proporção:</translation>
     </message>
@@ -429,7 +441,7 @@
         <translation>Parar de semear se ocioso por N minutos:</translation>
     </message>
     <message>
-        <location line="+8"/>
+        <location line="+9"/>
         <source>&amp;Idle:</source>
         <translation>Ocioso:</translation>
     </message>
@@ -439,7 +451,7 @@
         <translation>Conexões da Fonte</translation>
     </message>
     <message>
-        <location line="+7"/>
+        <location line="+8"/>
         <source>&amp;Maximum peers:</source>
         <translation>Máximo de fontes:</translation>
     </message>
@@ -742,27 +754,19 @@
         <translation></translation>
     </message>
     <message>
-        <location line="+35"/>
         <source>&amp;Add File...</source>
-        <translation>Adicionar Arquivo...</translation>
+        <translation type="obsolete">Adicionar Arquivo...</translation>
     </message>
     <message>
-        <location line="+3"/>
-        <source>Add</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location line="+3"/>
         <source>Add a torrent</source>
-        <translation>Adiciona um torrent</translation>
+        <translation type="obsolete">Adiciona um torrent</translation>
     </message>
     <message>
-        <location line="+3"/>
         <source>Ctrl+D</source>
-        <translation>Ctrl+D</translation>
+        <translation type="obsolete">Ctrl+D</translation>
     </message>
     <message>
-        <location line="+5"/>
+        <location line="+49"/>
         <source>&amp;New...</source>
         <translation>Novo...</translation>
     </message>
@@ -792,20 +796,44 @@
         <translation>Alt+Enter</translation>
     </message>
     <message>
-        <location line="+5"/>
         <source>&amp;Open Folder</source>
-        <translation>Abrir Pasta</translation>
+        <translation type="obsolete">Abrir Pasta</translation>
     </message>
     <message>
-        <location line="+3"/>
+        <location line="+8"/>
         <source>Open the torrent&apos;s folder</source>
         <translation>Abre a pasta do torrent</translation>
     </message>
     <message>
-        <location line="+3"/>
+        <location line="-30"/>
         <source>Ctrl+O</source>
         <translation>Ctrl+O</translation>
     </message>
+    <message>
+        <location line="-9"/>
+        <source>&amp;Open...</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+3"/>
+        <source>Open</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+3"/>
+        <source>Open a torrent</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+30"/>
+        <source>Open Fold&amp;er</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+6"/>
+        <source>Ctrl+E</source>
+        <translation type="unfinished"></translation>
+    </message>
     <message>
         <location line="+5"/>
         <source>&amp;Start</source>
@@ -856,6 +884,11 @@
         <source>Verify local data</source>
         <translation>Verifica os dados já baixados</translation>
     </message>
+    <message>
+        <location line="+3"/>
+        <source>Ctrl+V</source>
+        <translation type="unfinished"></translation>
+    </message>
     <message>
         <location line="+5"/>
         <source>&amp;Remove</source>
@@ -1106,8 +1139,17 @@
     </message>
     <message>
         <location line="+5"/>
+        <source>Open &amp;URL...</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+3"/>
+        <source>Ctrl+U</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
         <source>Add &amp;URL...</source>
-        <translation>Adicionar URL...</translation>
+        <translation type="obsolete">Adicionar URL...</translation>
     </message>
     <message>
         <location line="+5"/>
@@ -1247,7 +1289,7 @@ Adicione outra URL primária depois de uma linha em branco.</translation>
 <context>
     <name>MyApp</name>
     <message>
-        <location filename="../app.cc" line="+218"/>
+        <location filename="../app.cc" line="+223"/>
         <source>Transmission is a file-sharing program.  When you run a torrent, its data will be made available to others by means of upload.  You and you alone are fully responsible for exercising proper judgement and abiding by your local laws.</source>
         <translation>O Transmission é um programa de compartilhamento de arquivos que torna os dados de cada torrent executado disponíveis a outras pessoas por meio do upload. Você e apenas você é responsável por respeitar as leis locais do seu país.</translation>
     </message>
@@ -1269,11 +1311,15 @@ Adicione outra URL primária depois de uma linha em branco.</translation>
 </context>
 <context>
     <name>Options</name>
+    <message>
+        <source>Add Torrent</source>
+        <translation type="obsolete">Adicionar Torrent</translation>
+    </message>
     <message>
         <location filename="../options.cc" line="+22"/>
         <location line="+314"/>
-        <source>Add Torrent</source>
-        <translation>Adicionar Torrent</translation>
+        <source>Open Torrent</source>
+        <translation type="unfinished"></translation>
     </message>
     <message>
         <location line="-303"/>
@@ -1334,7 +1380,7 @@ Adicione outra URL primária depois de uma linha em branco.</translation>
 <context>
     <name>PrefsDialog</name>
     <message>
-        <location filename="../prefs-dialog.cc" line="+208"/>
+        <location filename="../prefs-dialog.cc" line="+186"/>
         <source>Use &amp;authentication</source>
         <translation>Usar autenticação</translation>
     </message>
@@ -1490,12 +1536,12 @@ Adicione outra URL primária depois de uma linha em branco.</translation>
     </message>
     <message>
         <location line="+5"/>
-        <location line="+358"/>
+        <location line="+365"/>
         <source>Status unknown</source>
         <translation>Estado desconhecido</translation>
     </message>
     <message>
-        <location line="-350"/>
+        <location line="-357"/>
         <source>&amp;Port for incoming connections:</source>
         <translation>Porta para conexões de entrada:</translation>
     </message>
@@ -1504,8 +1550,16 @@ Adicione outra URL primária depois de uma linha em branco.</translation>
         <source>Use UPnP or NAT-PMP port &amp;forwarding from my router</source>
         <translation>Usar encaminhamento UPnP ou NAT-PMP no meu roteador</translation>
     </message>
+    <message numerus="yes">
+        <location line="+311"/>
+        <source>&lt;i&gt;Blocklist contains %Ln rules&lt;/i&gt;</source>
+        <translation type="unfinished">
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+        </translation>
+    </message>
     <message>
-        <location line="-1"/>
+        <location line="-312"/>
         <source>Pick a &amp;random port every time Transmission is started</source>
         <translation>Usar uma porta aleatória quando iniciar</translation>
     </message>
@@ -1704,16 +1758,8 @@ Adicione outra URL primária depois de uma linha em branco.</translation>
         <source>Keep &amp;incomplete files in:</source>
         <translation>Manter arquivos incompletos em:</translation>
     </message>
-    <message numerus="yes">
-        <location line="+110"/>
-        <source>&lt;i&gt;Blocklist contains %Ln rules)</source>
-        <translation type="unfinished">
-            <numerusform></numerusform>
-            <numerusform></numerusform>
-        </translation>
-    </message>
     <message>
-        <location line="-112"/>
+        <location line="-2"/>
         <source>Save to &amp;Location:</source>
         <translation>Salvar em:</translation>
     </message>
@@ -1764,7 +1810,7 @@ Adicione outra URL primária depois de uma linha em branco.</translation>
         <translation>Internet</translation>
     </message>
     <message>
-        <location line="+27"/>
+        <location line="+28"/>
         <source>Not supported by remote sessions</source>
         <translation>Não suportado por sessões remotas</translation>
     </message>
@@ -1772,7 +1818,7 @@ Adicione outra URL primária depois de uma linha em branco.</translation>
 <context>
     <name>QObject</name>
     <message>
-        <location filename="../app.cc" line="-140"/>
+        <location filename="../app.cc" line="-145"/>
         <source>Invalid option</source>
         <translation>Opção inválida</translation>
     </message>
@@ -1813,7 +1859,7 @@ Adicione outra URL primária depois de uma linha em branco.</translation>
 <context>
     <name>Session</name>
     <message>
-        <location filename="../session.cc" line="+785"/>
+        <location filename="../session.cc" line="+824"/>
         <source>Add Torrent</source>
         <translation>Adicionar Torrent</translation>
     </message>
@@ -1920,7 +1966,7 @@ Adicione outra URL primária depois de uma linha em branco.</translation>
 <context>
     <name>Torrent</name>
     <message>
-        <location filename="../torrent.cc" line="+686"/>
+        <location filename="../torrent.cc" line="+699"/>
         <source>Waiting to verify local data</source>
         <translation>Verificação de dados locais em espera</translation>
     </message>
@@ -2068,7 +2114,7 @@ Adicione outra URL primária depois de uma linha em branco.</translation>
 <context>
     <name>TrMainWindow</name>
     <message>
-        <location filename="../mainwin.cc" line="+450"/>
+        <location filename="../mainwin.cc" line="+451"/>
         <source>Limit Download Speed</source>
         <translation>Limitar Download</translation>
     </message>
@@ -2159,12 +2205,11 @@ Down: %2</source>
 (baixar a %1 e enviar a %2)</translation>
     </message>
     <message>
-        <location line="+29"/>
         <source>Add Torrent</source>
-        <translation>Adicionar Torrent</translation>
+        <translation type="obsolete">Adicionar Torrent</translation>
     </message>
     <message>
-        <location line="+2"/>
+        <location line="+31"/>
         <source>Torrent Files (*.torrent);;All Files (*.*)</source>
         <translation>Arquivos Torrent (*.torrent);;Todos os Arquivos (*.*)</translation>
     </message>
@@ -2174,10 +2219,19 @@ Down: %2</source>
         <translation>Exibir diálogo de opções</translation>
     </message>
     <message>
-        <location line="+24"/>
-        <location line="+1"/>
         <source>Add URL or Magnet Link</source>
-        <translation>Adicionar URL ou link Magnet</translation>
+        <translation type="obsolete">Adicionar URL ou link Magnet</translation>
+    </message>
+    <message>
+        <location line="-5"/>
+        <source>Open Torrent</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+36"/>
+        <location line="+1"/>
+        <source>Open URL or Magnet Link</source>
+        <translation type="unfinished"></translation>
     </message>
     <message>
         <location line="+55"/>
@@ -2260,7 +2314,7 @@ Down: %2</source>
         <translation>O servidor respondeu pela ultima vez à %1 atrás</translation>
     </message>
     <message numerus="yes">
-        <location line="-573"/>
+        <location line="-580"/>
         <source>%L1 of %Ln Torrent(s)</source>
         <translation>
             <numerusform>%L1 of %Ln Torrent</numerusform>
