@@ -39,8 +39,8 @@ Torrent._MetaDataFields = [ 'addedDate', 'comment', 'creator', 'dateCreated',
 		'isPrivate', 'name', 'totalSize', 'pieceCount', 'pieceSize' ]
 
 Torrent._DynamicFields = [ 'downloadedEver', 'error', 'errorString', 'eta',
-    'haveUnchecked', 'haveValid', 'leftUntilDone', 'metadataPercentComplete', 'peersConnected',
-    'peersGettingFromUs', 'peersSendingToUs', 'rateDownload', 'rateUpload',
+    'haveUnchecked', 'haveValid', 'leftUntilDone', 'metadataPercentComplete', 'peers',
+    'peersConnected', 'peersGettingFromUs', 'peersSendingToUs', 'rateDownload', 'rateUpload',
     'recheckProgress', 'sizeWhenDone', 'status', 'trackerStats', 'desiredAvailable',
     'uploadedEver', 'uploadRatio', 'seedRatioLimit', 'seedRatioMode', 'downloadDir', 'isFinished' ]
 
@@ -94,33 +94,41 @@ Torrent.prototype =
 		this._element = element;
 		this._controller = controller;
 		controller._rows.push( element );
-		
+
 		// Create the 'name' <div>
 		var e = document.createElement( 'div' );
 		e.className = 'torrent_name';
 		top_e.appendChild( e );
 		element._name_container = e;
-		
-		// Create the 'progress details' <div>
+
+		// Create the 'peer details' <div>
 		e = document.createElement( 'div' );
-		e.className = 'torrent_progress_details';
+		e.className = 'torrent_peer_details';
 		top_e.appendChild( e );
-		element._progress_details_container = e;
+		element._peer_details_container = e;
+
+		//Create a progress bar container
+		top_a = document.createElement( 'div' );
+		top_a.className = 'torrent_progress_bar_container';
+		element._progress_bar_container = top_a;
 
 		// Create the 'in progress' bar
 		e = document.createElement( 'div' );
 		e.className = 'torrent_progress_bar incomplete';
 		e.style.width = '0%';
-		top_e.appendChild( e );
+		top_a.appendChild( e );
 		element._progress_complete_container = e;
-			
+
 		// Create the 'incomplete' bar (initially hidden)
 		e = document.createElement( 'div' );
 		e.className = 'torrent_progress_bar incomplete';
 		e.style.display = 'none';
-		top_e.appendChild( e );
+		top_a.appendChild( e );
 		element._progress_incomplete_container = e;
-		
+
+		//Add the progress bar container to the torrent
+		top_e.appendChild(top_a);
+
 		// Add the pause/resume button - don't specify the
 		// image or alt text until the 'refresh()' function
 		// (depends on torrent state)
@@ -131,16 +139,16 @@ Torrent.prototype =
 		top_e.appendChild( e );
 		element._pause_resume_button_image = image;
 		if (!iPhone) $(e).bind('click', function(e) { element._torrent.clickPauseResumeButton(e); });
-		
-		// Create the 'peer details' <div>
+
+		// Create the 'progress details' <div>
 		e = document.createElement( 'div' );
-		e.className = 'torrent_peer_details';
+		e.className = 'torrent_progress_details';
 		top_e.appendChild( e );
-		element._peer_details_container = e;
-		
+		element._progress_details_container = e;
+
 		// Set the torrent click observer
 		element.bind('click', function(e){ element._torrent.clickTorrent(e) });
-		
+
 		// Safari hack - first torrent needs to be moved down for some reason. Seems to be ok when
 		// using <li>'s in straight html, but adding through the DOM gets a bit odd.
 		if ($.browser.safari)
@@ -150,11 +158,11 @@ Torrent.prototype =
 
 		// Update all the labels etc
 		this.refresh(data);
-		
+
 		// insert the element
 		transferListParent.appendChild(top_e);
 	},
-	
+
 	initializeTorrentFilesInspectorGroup: function( fileListParent ) {
 		var e = document.createElement( 'ul' );
 		e.className = 'inspector_torrent_file_list inspector_group';
@@ -162,11 +170,11 @@ Torrent.prototype =
 		fileListParent.appendChild( e );
 		this._fileList = e;
 	},
-	
+
 	fileList: function() {
 		return $(this._fileList);
 	},
-	
+
 	buildTrackerStats: function(trackerStats) {
 		result = [];
 		for( var i=0, tracker; tracker=trackerStats[i]; ++i ) {
@@ -200,7 +208,7 @@ Torrent.prototype =
 	 *  S E T T E R S   /   G E T T E R S
 	 *
 	 *--------------------------------------------*/
-	
+
 	/* Return the DOM element for this torrent (a <LI> element) */
 	element: function() {
 		return this._element;
@@ -222,10 +230,16 @@ Torrent.prototype =
 	downloadTotal: function() { return this._download_total; },
 	hash: function() { return this._hashString; },
 	id: function() { return this._id; },
+	isActiveFilter: function() { return this.peersGettingFromUs() > 0
+					 || this.peersSendingToUs() > 0
+					 || this.webseedsSendingToUs() > 0
+					 || this.state() == Torrent._StatusChecking; },
 	isActive: function() { return this.state() != Torrent._StatusPaused; },
 	isDownloading: function() { return this.state() == Torrent._StatusDownloading; },
+	isFinished: function() { return this._isFinishedSeeding; },
 	isSeeding: function() { return this.state() == Torrent._StatusSeeding; },
 	name: function() { return this._name; },
+	webseedsSendingToUs: function() { return this._webseeds_sending_to_us; },
 	peersSendingToUs: function() { return this._peers_sending_to_us; },
 	peersGettingFromUs: function() { return this._peers_getting_from_us; },
 	needsMetaData: function(){ return this._metadataPercentComplete < 1 },
@@ -235,8 +249,7 @@ Torrent.prototype =
 		return ( this._sizeWhenDone - this._leftUntilDone ) / this._sizeWhenDone;
 	},
 	getPercentDoneStr: function() {
-		return Math.floor(100 * Math.ratio( 100 * ( this._sizeWhenDone - this._leftUntilDone ),
-		                           this._sizeWhenDone )) / 100;
+		return Transmission.fmt.percentString( 100 * this.getPercentDone() );
 	},
 	size: function() { return this._size; },
 	state: function() { return this._state; },
@@ -244,8 +257,7 @@ Torrent.prototype =
 		switch( this.state() ) {
 			case Torrent._StatusSeeding:        return 'Seeding';
 			case Torrent._StatusDownloading:    return 'Downloading';
-			case Torrent._StatusPaused:         return this._isFinishedSeeding ? 'Seeding complete'
-                                                                               : 'Paused';
+			case Torrent._StatusPaused:         return this.isFinished() ? 'Seeding complete' : 'Paused';
 			case Torrent._StatusChecking:       return 'Verifying local data';
 			case Torrent._StatusWaitingToCheck: return 'Waiting to verify';
 			default:                            return 'error';
@@ -269,13 +281,13 @@ Torrent.prototype =
 			default:                      return -1;
 		}
 	},
-	
+
 	/*--------------------------------------------
 	 *
 	 *  E V E N T   F U N C T I O N S
 	 *
 	 *--------------------------------------------*/
-	
+
 	/*
 	 * Process a click event on this torrent
 	 */
@@ -288,7 +300,7 @@ Torrent.prototype =
 		$('#jqContextMenu').hide();
 
 		var torrent = this;
-		
+
 		// 'Apple' button emulation on PC :
 		// Need settable meta-key and ctrl-key variables for mac emulation
 		var meta_key = event.metaKey;
@@ -297,35 +309,35 @@ Torrent.prototype =
 			meta_key = true;
 			ctrl_key = false;
 		}
-		
+
 		// Shift-Click - Highlight a range between this torrent and the last-clicked torrent
 		if (iPhone) {
 			if ( torrent.isSelected() )
 				torrent._controller.showInspector();
 			torrent._controller.setSelectedTorrent( torrent, true );
-		
+
 		} else if (event.shiftKey) {
 			torrent._controller.selectRange( torrent, true );
 			// Need to deselect any selected text
 			window.focus();
-		
+
 		// Apple-Click, not selected
 		} else if (!torrent.isSelected() && meta_key) {
 			torrent._controller.selectTorrent( torrent, true );
-			
+
 		// Regular Click, not selected
 		} else if (!torrent.isSelected()) {
 			torrent._controller.setSelectedTorrent( torrent, true );
-		
-		// Apple-Click, selected	
+
+		// Apple-Click, selected
 		} else if (torrent.isSelected() && meta_key) {
 			torrent._controller.deselectTorrent( torrent, true );
-			
+
 		// Regular Click, selected
 		} else if (torrent.isSelected()) {
 			torrent._controller.setSelectedTorrent( torrent, true );
 		}
-		
+
 		torrent._controller.setLastTorrentClicked(torrent);
 	},
 
@@ -357,7 +369,7 @@ Torrent.prototype =
 		this.refreshFileView();
 		this.refreshHTML( );
 	},
-	
+
 	refresh: function(data) {
 		this.refreshData( data );
 		this.refreshHTML( );
@@ -381,9 +393,11 @@ Torrent.prototype =
 		this._seed_ratio_mode         = data.seedRatioMode;
 		this._download_speed          = data.rateDownload;
 		this._upload_speed            = data.rateUpload;
+		this._peers                   = data.peers;
 		this._peers_connected         = data.peersConnected;
 		this._peers_getting_from_us   = data.peersGettingFromUs;
 		this._peers_sending_to_us     = data.peersSendingToUs;
+		this._webseeds_sending_to_us  = data.webseedsSendingToUs;
 		this._sizeWhenDone            = data.sizeWhenDone;
 		this._recheckProgress         = data.recheckProgress;
 		this._error                   = data.error;
@@ -423,10 +437,18 @@ Torrent.prototype =
 		return null;
 	},
 
+	formatUL: function() {
+		return 'UL: ' + Transmission.fmt.speedBps(this._upload_speed);
+	},
+	formatDL: function() {
+		return 'DL: ' + Transmission.fmt.speedBps(this._download_speed);
+	},
+
 	getPeerDetails: function()
 	{
 		var c;
 
+		var compact_mode = this._controller[Prefs._CompactDisplayState];
 		if(( c = this.getErrorMessage( )))
 			return c;
 
@@ -439,34 +461,25 @@ Torrent.prototype =
 				break;
 
 			case Torrent._StatusDownloading:
-				// 'Downloading from 36 of 40 peers - DL: 60.2 KiB/s UL: 4.3 KiB/s'
-				c = 'Downloading from ';
-				c += this.peersSendingToUs();
-				c += ' of ';
-				c += this._peers_connected;
-				c += ' peers - DL: ';
-				c += Math.formatBytes(this._download_speed);
-				c += '/s UL: ';
-				c += Math.formatBytes(this._upload_speed);
-				c += '/s';
+				var a = [ ];
+				if(!compact_mode)
+					a.push( 'Downloading from', this.peersSendingToUs(), 'of', this._peers_connected, 'peers', '-' );
+				a.push( this.formatDL(), this.formatUL() );
+				c = a.join(' ');
 				break;
 
 			case Torrent._StatusSeeding:
-				// 'Seeding to 13 of 22 peers - UL: 36.2 KiB/s'
-				c = 'Seeding to ';
-				c += this.peersGettingFromUs();
-				c += ' of ';
-				c += this._peers_connected;
-				c += ' peers - UL: ';
-				c += Math.formatBytes(this._upload_speed);
-				c += '/s';
+				if(compact_mode){
+					c = this.formatUL();
+				} else {
+					// 'Seeding to 13 of 22 peers - UL: 36.2 KiB/s'
+					c = [ 'Seeding to', this.peersGettingFromUs(), 'of', this._peers_connected, 'peers', '-', this.formatUL() ].join(' ');
+				}
 				break;
 
 			case Torrent._StatusChecking:
 				// 'Verifying local data (40% tested)'
-				c = 'Verifying local data (';
-				c += Math.roundWithPrecision( 100.0 * this._recheckProgress, 0 );
-				c += '% tested)';
+				c = [ 'Verifying local data (', Transmission.fmt.percentString( 100.0 * this._recheckProgress ), '% tested)' ].join('');
 				break;
 		}
 		return c;
@@ -478,9 +491,21 @@ Torrent.prototype =
 		var progress_details;
 		var root = this._element;
 		var MaxBarWidth = 100; // reduce this to make the progress bar shorter (%)
-		
+		var compact_mode = this._controller[Prefs._CompactDisplayState];
+		var compact = '';
+		if(compact_mode){
+			compact = ' compact';
+			root._peer_details_container.style.display = 'none';
+		} else {
+			root._peer_details_container.style.display = 'block';
+		}
+
+		root._progress_details_container.className = 'torrent_progress_details'+compact
+		root._progress_bar_container.className = 'torrent_progress_bar_container'+compact;
+		root._name_container.className = 'torrent_name'+compact;
+
 		setInnerHTML( root._name_container, this._name );
-		
+
 		// Add the progress bar
 		var notDone = this._leftUntilDone > 0;
 
@@ -490,57 +515,49 @@ Torrent.prototype =
 			notDone = false ;
 
 		if( this.needsMetaData() ){
-			var metaPercentComplete = this._metadataPercentComplete * 1000 / 100
-			progress_details = "Magnetized transfer - retrieving metadata (";
-			progress_details += metaPercentComplete;
-			progress_details += "%)";
+			var metaPercentComplete = this._metadataPercentComplete * 100;
+			progress_details = [ "Magnetized transfer - retrieving metadata (",
+			                     Transmission.fmt.percentString( metaPercentComplete ),
+			                     "%)" ].join('');
 
 			var empty = "";
 			if(metaPercentComplete == 0)
 				empty = "empty";
 
 			root._progress_complete_container.style.width = metaPercentComplete + "%";
-			root._progress_complete_container.className = 'torrent_progress_bar in_progress meta ' + empty;
+			root._progress_complete_container.className = 'torrent_progress_bar in_progress meta ' + empty+compact;
 			root._progress_incomplete_container.style.width = 100 - metaPercentComplete + "%"
-			root._progress_incomplete_container.className = 'torrent_progress_bar incomplete meta';
+			root._progress_incomplete_container.className = 'torrent_progress_bar incomplete compact meta'+compact;
 			root._progress_incomplete_container.style.display = 'block';
-
 		}
 		else if( notDone )
 		{
-			var eta = '';
-			
-			if( this.isActive( ) )
-			{
-				eta = ' - ';
-				if (this._eta < 0 || this._eta >= Torrent._InfiniteTimeRemaining )
-					eta += 'remaining time unknown';
-				else
-					eta += Math.formatSeconds(this._eta) + ' remaining';
-			}
-			
 			// Create the 'progress details' label
 			// Eg: '101 MiB of 631 MiB (16.02%) - 2 hr remaining'
-			c = Math.formatBytes( this._sizeWhenDone - this._leftUntilDone );
-			c += ' of ';
-			c += Math.formatBytes( this._sizeWhenDone );
-			c += ' (';
-			c += this.getPercentDoneStr();
-			c += '%)';
-			c += eta;
-			progress_details = c;
-		
+
+			c = [ Transmission.fmt.size( this._sizeWhenDone - this._leftUntilDone ),
+			      ' of ', Transmission.fmt.size( this._sizeWhenDone ),
+			      ' (', this.getPercentDoneStr(), '%)' ];
+			if( this.isActive( ) ) {
+				c.push( ' - ' );
+				if (this._eta < 0 || this._eta >= Torrent._InfiniteTimeRemaining )
+					c.push( 'remaining time unknown' );
+				else
+					c.push( Transmission.fmt.timeInterval(this._eta) + ' remaining' );
+			}
+			progress_details = c.join('');
+
 			// Figure out the percent completed
-			var css_completed_width = Math.floor( this.getPercentDone() * 100 * MaxBarWidth ) / 100;
-			
+			var css_completed_width = ( this.getPercentDone() * MaxBarWidth ).toTruncFixed( 2 );
+
 			// Update the 'in progress' bar
 			e = root._progress_complete_container;
-			c = 'torrent_progress_bar';
-			c += this.isActive() ? ' in_progress' : ' incomplete_stopped';
-			if(css_completed_width === 0) { c += ' empty'; }
-			e.className = c;
+			c = [ 'torrent_progress_bar'+compact,
+			      this.isActive() ? 'in_progress' : 'incomplete_stopped' ];
+			if(css_completed_width === 0) { c.push( 'empty' ); }
+			e.className = c.join(' ');
 			e.style.width = css_completed_width + '%';
-			
+
 			// Update the 'incomplete' bar
 			e = root._progress_incomplete_container;
 			e.className = 'torrent_progress_bar incomplete'
@@ -549,32 +566,36 @@ Torrent.prototype =
 		}
 		else
 		{
-			var eta = '';
+			// Create the 'progress details' label
 
-			if( this.isActive( ) && this.seedRatioLimit( ) > 0 )
+			if( this._size == this._sizeWhenDone )
 			{
-				eta = ' - ';
-				if (this._eta < 0 || this._eta >= Torrent._InfiniteTimeRemaining )
-					eta += 'remaining time unknown';
-				else
-					eta += Math.formatSeconds(this._eta) + ' remaining';
+				// seed: '698.05 MiB'
+				c = [ Transmission.fmt.size( this._size ) ];
+			}
+			else
+			{
+				// partial seed: '127.21 MiB of 698.05 MiB (18.2%)'
+				c = [ Transmission.fmt.size( this._sizeWhenDone ), ' of ', Transmission.fmt.size( this._size ),
+				      ' (', Transmission.fmt.percentString( 100.0 * this._sizeWhenDone / this._size ), '%)' ];
 			}
 
-			// Create the 'progress details' label
-			// Eg: '698.05 MiB, uploaded 8.59 GiB (Ratio: 12.3)'
-			c = Math.formatBytes( this._size );
-			c += ', uploaded ';
-			c += Math.formatBytes( this._upload_total );
-			c += ' (Ratio ';
-			if(this._upload_ratio > -1)
-				c += Math.round(this._upload_ratio*100)/100;
-			else if(this._upload_ratio == -2)
-				c += 'Inf';
-			else
-				c += '0';
-			c += ')';
-			c += eta;
-			progress_details = c;
+			// append UL stats: ', uploaded 8.59 GiB (Ratio: 12.3)'
+			c.push( ', uploaded ', Transmission.fmt.size( this._upload_total ),
+			        ' (Ratio ', Transmission.fmt.ratioString( this._upload_ratio ), ')' );
+
+			// maybe append remaining time
+			if( this.isActive( ) && this.seedRatioLimit( ) > 0 )
+			{
+				c.push(' - ');
+
+				if (this._eta < 0 || this._eta >= Torrent._InfiniteTimeRemaining )
+					c.push( 'remaining time unknown' );
+				else
+					c.push( Transmission.fmt.timeInterval(this._eta), ' remaining' );
+			}
+
+			progress_details = c.join('');
 
 			var status = this.isActive() ? 'complete' : 'complete_stopped';
 
@@ -605,23 +626,38 @@ Torrent.prototype =
 			e = root._progress_complete_container;
 			e.className = 'torrent_progress_bar ' + status;
 		}
-		
+
+		var hasError = this.getErrorMessage( ) != undefined;
 		// Update the progress details
+		if(compact_mode){
+			progress_details = this.getPeerDetails();
+			$(root._progress_details_container).toggleClass('error',hasError);
+		} else {
+			$(root._peer_details_container).toggleClass('error',hasError);
+		}
 		setInnerHTML( root._progress_details_container, progress_details );
-		
+
+		if( compact ){
+			var width = root._progress_details_container.offsetLeft - root._name_container.offsetLeft;
+			root._name_container.style.width = width + 'px';
+		}
+		else {
+			root._name_container.style.width = '100%';
+		}
+
 		// Update the peer details and pause/resume button
 		e = root._pause_resume_button_image;
 		if ( this.state() === Torrent._StatusPaused ) {
 			e.alt = 'Resume';
-			e.className = "torrent_resume";
+			e.className = "torrent_resume"+compact;
 		} else {
 			e.alt = 'Pause';
-			e.className = "torrent_pause";
+			e.className = "torrent_pause"+compact;
 		}
-		
+
 		setInnerHTML( root._peer_details_container, this.getPeerDetails( ) );
-	
-		this.refreshFileView( );	
+
+		this.refreshFileView( );
 	},
 
 	refreshFileView: function() {
@@ -665,9 +701,12 @@ Torrent.prototype =
 	test: function( filter, search )
 	{
 		var pass = false;
-		
+
 		switch( filter )
 		{
+			case Prefs._FilterActive:
+				pass = this.isActiveFilter();
+				break;
 			case Prefs._FilterSeeding:
 				pass = this.isSeeding();
 				break;
@@ -677,17 +716,20 @@ Torrent.prototype =
 			case Prefs._FilterPaused:
 				pass = !this.isActive();
 				break;
+			case Prefs._FilterFinished:
+				pass = this.isFinished();
+				break;
 			default:
 				pass = true;
 				break;
 		}
-		
+
 		if( !pass )
 			return false;
-		
+
 		if( !search || !search.length )
 			return pass;
-		
+
 		return this._name_lc.indexOf( search.toLowerCase() ) !== -1;
 	}
 };
@@ -757,10 +799,10 @@ Torrent.sortTorrents = function( torrents, sortMethod, sortDirection )
 			console.warn( "unknown sort method: " + sortMethod );
 			break;
 	}
-	
+
 	if( sortDirection === Prefs._SortDescending )
 		torrents.reverse( );
-	
+
 	return torrents;
 };
 
@@ -823,7 +865,7 @@ TorrentFile.prototype = {
 		li.appendChild(pri_div);
 		li.appendChild(file_div);
 		li.appendChild(prog_div);
-		
+
 		this._element = li;
 		this._priority_control = pri_div;
 		this._progress = $(prog_div);
@@ -841,7 +883,7 @@ TorrentFile.prototype = {
 	isEditable: function () {
 		return (this._torrent._file_model.length>1) && !this.isDone();
 	},
-	
+
 	readAttributes: function(file_data) {
 		if( file_data.index !== undefined && file_data.index !== this._index ) {
 			this._index = file_data.index;
@@ -864,15 +906,15 @@ TorrentFile.prototype = {
 			this._dirty = true;
 		}
 	},
-	
+
 	element: function() {
 		return $(this._element);
 	},
-	
+
 	domElement: function() {
 		return this._element;
 	},
-	
+
 	setPriority: function(prio) {
 		if (this.isEditable()) {
 			var cmd;
@@ -886,23 +928,23 @@ TorrentFile.prototype = {
 			this._torrent._controller.changeFileCommand( cmd, this._torrent, this );
 		}
 	},
-	
+
 	setWanted: function(wanted, process) {
 		this._dirty = true;
 		this._wanted = wanted;
 		if(!iPhone)
-		  this.element().toggleClass( 'skip', !wanted );
+			this.element().toggleClass( 'skip', !wanted );
 		if (process) {
 			var command = wanted ? 'files-wanted' : 'files-unwanted';
 			this._torrent._controller.changeFileCommand(command, this._torrent, this);
 		}
 	},
-	
+
 	toggleWanted: function() {
 		if (this.isEditable())
 			this.setWanted( !this._wanted, true );
 	},
-	
+
 	refreshHTML: function() {
 		if( this._dirty ) {
 			this._dirty = false;
@@ -911,40 +953,40 @@ TorrentFile.prototype = {
 			this.refreshPriorityHTML();
 		}
 	},
-	
+
 	refreshProgressHTML: function() {
-		var c = Math.formatBytes(this._done);
-		c += ' of ';
-		c += Math.formatBytes(this._size);
-		c += ' (';
-		c += Math.ratio(100 * this._done, this._size);
-		c += '%)';
+		var c = [ Transmission.fmt.size(this._done),
+		          ' of ',
+		          Transmission.fmt.size(this._size),
+		          ' (',
+		          this._size ? Transmission.fmt.percentString(100 * this._done / this._size) : '100',
+		          '%)' ].join('');
 		setInnerHTML(this._progress[0], c);
 	},
-	
+
 	refreshWantedHTML: function() {
 		var e = this.domElement();
-		var c = e.classNameConst;
-		if(!this._wanted) { c += ' skip'; }
-		if(this.isDone()) { c += ' complete'; }
-		e.className = c;
+		var c = [ e.classNameConst ];
+		if(!this._wanted) { c.push( 'skip' ); }
+		if(this.isDone()) { c.push( 'complete' ); }
+		e.className = c.join(' ');
 	},
-	
+
 	refreshPriorityHTML: function() {
 		var e = this._priority_control;
-		var c = e.classNameConst;
+		var c = [ e.classNameConst ];
 		switch( this._prio ) {
-			case 1: c += ' high'; break;
-			case -1: c += ' low'; break;
-			default: c += ' normal'; break;
+			case 1  : c.push( 'high'   ); break;
+			case -1 : c.push( 'low'    ); break;
+			default : c.push( 'normal' ); break;
 		}
-		e.className = c;
+		e.className = c.join(' ');
 	},
-	
+
 	fileWantedControlClicked: function(event) {
 		this.toggleWanted();
 	},
-	
+
 	filePriorityControlClicked: function(event, element) {
 		var x = event.pageX;
 		while (element !== null) {
